#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''  
   Copyright (c) 2019 Yahoo Japan Corporation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
'''

import sys
import re
from collections import defaultdict
import argparse

ap = argparse.ArgumentParser()
ap.add_argument('-l', '--labeledfile', type=str, required=True)
ap.add_argument('-m', '--mapfile', type=str, required=True)
ap.add_argument('--sep', type=str, default="\t")
ap.add_argument('--pos', type=str, default="__label__POS")
ap.add_argument('--neg', type=str, default="__label__NEG")
ap.add_argument('--causalitylbl', type=str, default="CAUSALITY")
ap.add_argument('--causelbl', type=str, default="CAUSE")
ap.add_argument('--effectlbl', type=str, default="EFFECT")
ap.add_argument('--titlelbl', type=str, default="TITLE")
ap.add_argument('--entsymbol', type=str, default="__ent__")
ap.add_argument('--causesymbol', type=str, default="__CAUSE__")
ap.add_argument('--effectsymbol', type=str, default="__EFFECT__")
ap.add_argument('--end_of_context', type=str, default=" __EOC__ ")
ap.add_argument('--newlinesymbol', type=str, default="__||__")
ap.add_argument('--csep', type=str, default="\t")
ap.add_argument('--lblsep', type=str, default=":")
args = ap.parse_args()

def read_lbld():
    lbldb = dict()
    with open(args.labeledfile, 'r') as f:
        for l in f:
            l = l.rstrip("\n")
            lbl, wid1, wid2 = l.split(args.sep)
            lbldb[wid1 + args.sep + wid2] = args.pos if lbl == '1' else args.neg
    return lbldb

def read_map():
    mapdb = dict()
    with open(args.mapfile, 'r') as f:
        for l in f:
            l = l.rstrip("\n")
            ent1, ent2, sharp, wid1, wid2 = l.split(args.sep)
            mapdb[ent1 + args.sep + ent2] = wid1 + args.sep + wid2
    return mapdb

def process_contexts(ctxts, cause_ent, effect_ent, target_symbol):
    ctxts = ctxts.lower()
    cause_ent = cause_ent.lower()
    effect_ent = effect_ent.lower()
    ctxts = ctxts.replace(args.entsymbol, target_symbol)
    ctxts = ctxts.replace(cause_ent, args.causesymbol)
    ctxts = ctxts.replace(effect_ent, args.effectsymbol)
    ctxts = ctxts.replace(args.newlinesymbol, " %s " % args.newlinesymbol)
    ctxts = args.end_of_context.join(ctxts.split(args.csep))
    return ctxts

def process(lbldb, mapdb):
    entpair2contexts = defaultdict(list) # ent1 ent2 -> context context ...
    causality, cause_ent, effect_ent = None, None, None
    for l in sys.stdin:
        l = l.rstrip("\n")
        if l.startswith(args.titlelbl): continue
        elif l.startswith(args.causalitylbl):
            tmplbl, causality = l.split(args.lblsep, 1)
            cause_ent, effect_ent = causality.split(args.sep)
        elif l.startswith(args.causelbl):
            tmplbl, cause = l.split(args.lblsep, 1)
            ctxts = process_contexts(cause, cause_ent, effect_ent, args.causesymbol)
            entpair2contexts[causality].append(ctxts)
        elif l.startswith(args.effectlbl):
            tmplbl, effect = l.split(args.lblsep, 1)
            ctxts = process_contexts(effect, cause_ent, effect_ent, args.effectsymbol)
            entpair2contexts[causality].append(ctxts)
        else:
            print("Unknown label:%s" % l, file=sys.stderr)
            sys.exit(1)
    for causality, ctxts_li in entpair2contexts.items():
        widpair = mapdb.get(causality, None)
        if not widpair:
            print("No WID pair found:%s" % causality, file=sys.stderr)
            sys.exit(1)
        label = lbldb.get(widpair, None)
        if not label:
            print("No label found:%s ... %s" % (widpair, causality), file=sys.stderr)
            sys.exit(1)
        ctxts = args.end_of_context.join(ctxts_li)
        print("%s\t%s\t%s\t%s" % (label, widpair, causality, ctxts))
        del lbldb[widpair]
    if lbldb:
        for widpair, label in lbldb.items():
            print("%s\t%s\t%s\t%s" % (label, widpair, "-\t-", "-"))
            print("No feat found:%s" % widpair, file=sys.stderr)

if __name__ == "__main__":
    lbldb = read_lbld()
    mapdb = read_map()
    process(lbldb, mapdb)

