#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''  
   Copyright (c) 2019 Yahoo Japan Corporation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
'''

import sys
import xml.etree.ElementTree as ET
from flashtext import KeywordProcessor
from collections import defaultdict
import argparse

ap = argparse.ArgumentParser()
ap.add_argument('-p', '--pairfile', type=str, required=True)
ap.add_argument('-s', '--sep', type=str, default="\t")
ap.add_argument('--tokensep', type=str, default=" ")
ap.add_argument('--pagestart', type=str, default="<page>")
ap.add_argument('--pageend', type=str, default="</page>")
ap.add_argument('--titlepat', type=str, default=".//title")
ap.add_argument('--textpat', type=str, default=".//text")
ap.add_argument('--case_sensitive', type=bool, default=False)
ap.add_argument('--causelabel', type=str, default="c")
ap.add_argument('--effectlabel', type=str, default="e")
ap.add_argument('--target_ent_symbol', type=str, default="__ENT__")
ap.add_argument('--paircausecolumn', type=int, default=0)
ap.add_argument('--paireffectcolumn', type=int, default=1)
ap.add_argument('--newlinesymbol', type=str, default="__||__")
ap.add_argument('--no_infobox', action="store_true")
ap.add_argument('--infoboxstart', type=str, default="{{infobox")
ap.add_argument('--openparen', type=str, default="{")
ap.add_argument('--closeparen', type=str, default="}")
ap.add_argument('--window', type=int, default=100)
ap.add_argument('--left_context_trim_symbol', type=str, default="==__||__")
ap.add_argument('--right_context_trim_symbol', type=str, default="__||__==")
ap.add_argument('--allow_no_context', action="store_true")
args = ap.parse_args()

infoboxstartlen = len(args.infoboxstart)

def cause_effect_table():
    table = defaultdict(dict)
    with open(args.pairfile, 'r') as f:
        for l in f:
            li = l.strip().split(args.sep)
            cause = li[args.paircausecolumn]
            effect = li[args.paireffectcolumn]
            table[cause][effect] = args.effectlabel
            table[effect][cause] = args.causelabel
    return table

def prepare_keyword_processor(cetable):
    keyword_processor = KeywordProcessor(case_sensitive=args.case_sensitive)
    for ent in cetable.keys():
        keyword_processor.add_keyword(ent)
    return keyword_processor

def remove_infobox(string):
    string_orig = string
    while string:
        begin_ind = string.lower().find(args.infoboxstart)
        if begin_ind == -1: return string
        from_ = begin_ind + infoboxstartlen
        end_ind = None
        cnt = 0
        for i in range(from_, len(string)):
            c = string[i]
            if c == args.openparen: cnt += 1
            elif c == args.closeparen: cnt -= 1
            if cnt == -2:
                end_ind = i
                break
        if end_ind is None:
            return string
        string = string[0:begin_ind] + string[end_ind+1:-1]
    return None

def linearize_text(string):
    return string.replace("\t", " ").replace("\n", args.newlinesymbol)

def extr_entities_contexts(entities, text):
    ent2contexts = defaultdict(list)
    kpspan = KeywordProcessor()
    for ent in entities:
        kpspan.add_keyword(ent)
    entspans = kpspan.extract_keywords(text, span_info=True)
    for (ent, begin, end) in entspans:
        leftside = text[:begin].strip()
        leftside_tmpli = leftside.rsplit(args.left_context_trim_symbol, 1)
        if len(leftside_tmpli) > 1:
            leftside = leftside_tmpli[1]
        leftside_li = leftside.split(args.tokensep)[-args.window:]
        rightside = text[end:].strip()
        rightside = rightside.split(args.right_context_trim_symbol, 1)[0]
        rightside_li = rightside.split(args.tokensep)[:args.window]
        context = args.tokensep.join(leftside_li) + \
                args.tokensep + args.target_ent_symbol + args.tokensep + \
                args.tokensep.join(rightside_li)
        ent2contexts[ent].append(context)
    return ent2contexts

def identify_causalities(entities, cetable, title):
    result_entities = set()
    result_causalities = set()
    for ent1 in entities:
        if ent1 not in cetable: continue
        for ent2 in entities:
            if ent1 == ent2: continue
            if ent2 not in cetable[ent1]: continue
            if ent1 != title and ent2 != title: continue
            result_entities.add(ent1)
            result_entities.add(ent2)
            lbl = cetable[ent1][ent2]
            if lbl == args.effectlabel:
                result_causalities.add((ent1, ent2))
            elif lbl == args.causelabel:
                result_causalities.add((ent2, ent1))
            else:
                print("Unknown cause/effect label:%s" % lbl, file=sys.stderr)
                sys.exit(1)
    return (result_entities, result_causalities)

def process(xml, kp, cetable):
    root = ET.fromstring(''.join(xml))
    title = root.find(args.titlepat).text.strip()
    if title not in cetable: return None
    text = root.find(args.textpat).text
    if not text: return None
    if args.no_infobox:
        text = remove_infobox(text)
        if not text: return None
    text = linearize_text(text)
    extracted = set(kp.extract_keywords(text))
    if not extracted: return None
    extracted.add(title)
    entities, causalities = identify_causalities(extracted, cetable, title)
    #entities, causalities = identify_causalities(extracted, cetable)
    if not entities or not causalities: return None
    ent2contexts = extr_entities_contexts(entities, text)
    if not ent2contexts: return None
    result = []
    result.append("TITLE:" + title)
    for (cause, effect) in causalities:
        cause_contexts = ent2contexts.get(cause, [])
        if not cause_contexts and cause != title:
            if args.allow_no_context: continue
            print("No cause context:%s" % cause, file=sys.stderr)
            print("Ent2contexts:", file=sys.stderr)
            print(ent2contexts, file=sys.stderr)
            print("Entities:", file=sys.stderr)
            print(entities, file=sys.stderr)
            print("Causalities:", file=sys.stderr)
            print(causalities, file=sys.stderr)
            sys.exit(1)
        effect_contexts = ent2contexts.get(effect, [])
        if not effect_contexts and effect != title:
            if args.allow_no_context: continue
            print("No effect context:%s" % effect, file=sys.stderr)
            print("Ent2contexts:", file=sys.stderr)
            print(ent2contexts, file=sys.stderr)
            print("Entities:", file=sys.stderr)
            print(entities, file=sys.stderr)
            print("Causalities:", file=sys.stderr)
            print(causalities, file=sys.stderr)
            sys.exit(1)
        result.append("CAUSALITY:" + cause + "\t" + effect)
        result.append("CAUSE:" + "\t".join(cause_contexts))
        result.append("EFFECT:" + "\t".join(effect_contexts))
    if result:
        print("\n".join(result))
    return None

def main():
    cetable = cause_effect_table()
    kp = prepare_keyword_processor(cetable)
    xml = []
    record_flag = False
    for l in sys.stdin:
        if record_flag:
            xml.append(l)
        if args.pagestart in l:
            record_flag = True
            xml.append(l)
        elif args.pageend in l:
            record_flag = False
            process(xml, kp, cetable)
            xml = []

if __name__ == "__main__":
    main()

